# *
# *	Copyright (c) 2000 Alberto Reggiori / <alberto.reggiori@jrc.it>
# *	ISIS/RIT, Joint Research Center Ispra (I)
# *
# * NOTICE
# *
# * This product is distributed under a BSD/ASF like license as described in the 'LICENSE'
# * file you should have received together with this source code. If you did not get a
# * a copy of such a license agreement you can pick up one at:
# *
# *     http://xml.jrc.it/RDFStore/LICENSE
# *
# *
package RDF::Store::Statement;
{
use Carp;
use RDF::Statement;
use RDF::Store::Resource;

@RDF::Store::Statement::ISA = qw( RDF::Store::Resource RDF::Statement );

sub new {
    my $self = $_[0]->SUPER::new();
	
    croak "Subject ".$_[1]." is not instance of RDF::Resource"
                unless( (not(defined $_[1])) ||
                        ($_[1]->isa('RDF::Resource')) );
    croak "Predicate ".$_[2]." is not instance of RDF::Resource"
                unless( (not(defined $_[2])) ||
                        ($_[2]->isa('RDF::Resource')) );
    croak "Object ".$_[3]." is not instance of RDF::RDFNode"
                unless( (not(defined $_[3])) ||
                        ($_[3]->isa('RDF::RDFNode')) );

    $self->{subj} = $_[1];
    $self->{pred} = $_[2];
    $self->{obj} = $_[3];

    bless $self,$_[0];
};

sub subject {
	return $_[0]->{subj};
};

sub predicate {
	return $_[0]->{pred};
};

sub object {
	return $_[0]->{obj};
};

# Properties: digest can be constructed given the digests of the pred, subj, obj
# Permutations of the s, p, o return different digests
# see http://nestroy.wi-inf.uni-essen.de/rdf/sum_rdf_api/#K31
sub getDigest {
	if(not(defined $_[0]->{digest})) {
		my $s = $_[0]->{subj}->getDigest()->getDigestBytes();
		my $p = $_[0]->{pred}->getDigest()->getDigestBytes();
		my $o = $_[0]->{obj}->getDigest()->getDigestBytes();

		my $b = ${$s} . ${$p};
		if($_[0]->{obj}->isa("RDF::Resource")) {
			$b .= ${$o};
                } else { # rotate by one byte
			my @torotate = split(//,${$o});
			push @torotate,shift @torotate; #rotate
			$b .= join(//,@torotate);
                };

                $_[0]->{digest} = RDF::Digest::Util::computeDigest($_[0]->{algorithm},$b)
                        or croak "Cannot compute Digest for statement ",$_[0]->{label};
        };
        return $_[0]->{digest};
};

sub getURI {
        return "uuid:rdf:".$_[0]->{algorithm}."-".RDF::Digest::Util::toHexString($_[0]->getDigest());
};

sub getLabel {
        return $_[0]->getURI();
};

sub node2string {
	croak "Node ".$_[1]." is not instance of RDF::RDFNode"
                unless((defined $_[1]) && ($_[1]->isa('RDF::RDFNode')));

	if($_[1]->isa("RDF::Literal")) {
        	return "literal(\"".$_[1]->getLabel() ."\")";
	} elsif($_[1]->isa("RDF::Statement")) {
        	return $_[1]->toString();
      	} else { # Resource
        	return "\"".$_[1]->getLabel() ."\"";
	};
};

sub toString {
        return "triple(". 	$_[0]->node2string($_[0]->{subj}).", ".
				$_[0]->node2string($_[0]->{pred}).", ".
				$_[0]->node2string($_[0]->{obj}).")";
};

sub equals {
        if ($_[0] == $_[1]) {
                return 1;
        };
        if ( (not(defined $_[1])) || !($_[1]->isa("RDF::Statement")) ) {
                return 0;
        };

	# we alwasy assume that is Digestible
        return $_[0]->SUPER::equals($_[1]);
};

1;
};

__END__

=head1 NAME

RDF::Store::Statement - implementation of the Statement RDF API from Sergey Melnik (see http://www-db.stanford.edu/~melnik/rdf/api.html)

=head1 SYNOPSIS

  use RDF::Store::Statement;
  use RDF::Store::Literal;
  use RDF::Store::Resource;
  my $statement = new RDF::Store::Statement(
  				new RDF::Store::Resource("http://pen.jrc.it/idex.html"),
  				new RDF::Store::Resource("author","http://purl.org/schema/1.0#"),
  				new RDF::Store::Literal("Alberto Reggiori")
				);


=head1 DESCRIPTION

An RDF::Statement implementation using Digested URIs.

=head1 SEE ALSO

RDF::Statement(3) Digest(3) RDF::Store::RDFNode(3)

=head1 AUTHOR

	Alberto Reggiori <alberto.reggiori@jrc.it>
